//+------------------------------------------------------------------+
//|                                                     Rational.mqh |
//|                                 (c) 2011.04.03, Vladimir Gomonov |
//|                                            MetaDriver@rambler.ru |
//+------------------------------------------------------------------+
#property copyright "(c) 2011.04.03, Vladimir Gomonov"
#property link      "MetaDriver@rambler.ru"
#include <Math\gcd.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct LongRational
  {
   long              n;
   long              m;
   void ErrDZ() { Print("LongRational error: zero-denominator!"); }
   void Neg() { n=-n; }
   void Norm() { long d=gcd(n,m); n/=d; m/=d; if (m<0) { n=-n; m=-m; } }
   void Assign(long a,long b) { if (b==0) { n=0; m=1; ErrDZ(); } if (b<0) { n=-a; m=-b; } else { n=a; m=b; } }
   void nAssign(long a,long b) { Assign(a,b); Norm(); }
   void Assign(long a) { Assign(a,1); }  // {n=a;m=1;}
   void Add(LongRational &a) { if(m==a.m) n+=a.n; else { n*=a.m; n+=m*a.n; m*=a.m; } }
   void nAdd(LongRational &a) { Add(a); Norm(); }
   void Add(long a) { n+=a*m; }
   void nAdd(long a) { Add(a); Norm(); }
   void Sub(LongRational &a) { if(m==a.m) n-=a.n; else { n*=a.m; n-=m*a.n; m*=a.m; } }
   void nSub(LongRational &a) { Sub(a); Norm(); }
   void Sub(long a) { n-=a*m; }
   void nSub(long a) { Sub(a); Norm(); }
   void Mul(LongRational &a) { n*=a.n; m*=a.m; }
   void nMul(LongRational &a) { Mul(a); Norm(); }
   void Mul(long a) { n*=a; }
   void nMul(long a) { Mul(a); Norm(); }
   void Div(LongRational &a) { n*=a.m; m*=a.n; }
   void nDiv(LongRational &a) { Div(a); Norm(); }
   void Div(long a) { m*=a; }
   void nDiv(long a) { Div(a); Norm(); }
   string ToString() {return "("+IntegerToString(n)+"/"+IntegerToString(m)+")";}
  };

struct IntRational
  {
   int              n;
   int              m;
   void ErrDZ() { Print("IntRational error: zero-denominator!"); }
   void Neg() { n=-n; }
   void Norm() { int d=(int)gcd(n,m); n/=d; m/=d; if (m<0) { n=-n; m=-m; } }
   void Assign(int a,int b) { if (b==0) { n=0; m=1; ErrDZ(); } if (b<0) { n=-a; m=-b; } else { n=a; m=b; } }
   void nAssign(int a,int b) { Assign(a,b); Norm(); }
   void Assign(int a) { Assign(a,1); }  // {n=a;m=1;}
   void Add(IntRational &a) { if(m==a.m) n+=a.n; else { n*=a.m; n+=m*a.n; m*=a.m; } }
   void nAdd(IntRational &a) { Add(a); Norm(); }
   void Add(int a) { n+=a*m; }
   void nAdd(int a) { Add(a); Norm(); }
   void Sub(IntRational &a) { if(m==a.m) n-=a.n; else { n*=a.m; n-=m*a.n; m*=a.m; } }
   void nSub(IntRational &a) { Sub(a); Norm(); }
   void Sub(int a) { n-=a*m; }
   void nSub(int a) { Sub(a); Norm(); }
   void Mul(IntRational &a) { n*=a.n; m*=a.m; }
   void nMul(IntRational &a) { Mul(a); Norm(); }
   void Mul(int a) { n*=a; }
   void nMul(int a) { Mul(a); Norm(); }
   void Div(IntRational &a) { n*=a.m; m*=a.n; }
   void nDiv(IntRational &a) { Div(a); Norm(); }
   void Div(int a) { m*=a; }
   void nDiv(int a) { Div(a); Norm(); }
   string ToString() {return "("+IntegerToString(n)+"/"+IntegerToString(m)+")";}
  };

